/*************************************************************************
 *  Compilation:  javac LinearSearch.java
 *  Execution:    java LinearSearch whitelist.txt < input.txt
 *  Data files:   http://algs4.cs.princeton.edu/11model/tinyW.txt
 *                http://algs4.cs.princeton.edu/11model/tinyT.txt
 *                http://algs4.cs.princeton.edu/11model/largeW.txt
 *                http://algs4.cs.princeton.edu/11model/largeT.txt
 *
 *  % java LinearSearch tinyW.txt < tinyT.txt
 *  50
 *  99
 *  13
 *
 *  % java LinearSearch largeW.txt < largeT.txt | more
 *  499569
 *  984875
 *  295754
 *  207807
 *  140925
 *  161828
 *  [3,675,966 total values]
 *  
 *************************************************************************/

import java.util.Arrays;

/**
 *  The <tt>LinearSearch</tt> class provides a static method for binary
 *  searching for an integer in a sorted array of integers.
 *  <p>
 *  The <em>rank</em> operations takes logarithmic time in the worst case.
 *  <p>
 *  For additional documentation, see <a href="http://algs4.cs.princeton.edu/11model">Section 1.1</a> of
 *  <i>Algorithms, 4th Edition</i> by Robert Sedgewick and Kevin Wayne.
 *
 *  @author Robert Sedgewick
 *  @author Kevin Wayne
 *  @author Bob Muller, converted SW's BinarySearch to LinearSearch.
 */
public class LinearSearch {

    /**
     * This class should not be instantiated.
     */
    private LinearSearch() { }

    /**
     * Searches for the integer key in the UNSORTED array a[].
     * @param key the search key
     * @param a the array of integers, must be sorted in ascending order
     * @return index of key in array a[] if present; -1 if not present
     */
    public static int rank(int key, int[] a) {

        for(int i = 0; i < a.length; i++) {
            int compareResult = key - a[i];
            if (compareResult == 0) return i;
        }
        return -1;
    }

    /**
     * Reads in a sequence of integers from the whitelist file, specified as
     * a command-line argument. Reads in integers from standard input and
     * prints to standard output those integers that do *not* appear in the 
     * file.
     */
    public static void main(String[] args) {

        // read the integers from a file
        In in = new In(args[0]);
        int[] whitelist = in.readAllInts();

        int count = 0;

        // read integer key from standard input; count those not in whitelist
        //
        Stopwatch sw = new Stopwatch();
        while (!StdIn.isEmpty()) {
            int key = StdIn.readInt();
            if (rank(key, whitelist) == -1) count++;
        }
        System.out.println("count= " + count + ", time=" + sw.elapsedTime());
    }
}